@@ -3,6 +3,7 @@
 import urllib
 import urllib2
 import cookielib
+from threading import Thread
 from BeautifulSoup import BeautifulSoup
 from BeautifulSoup import BeautifulStoneSoup
 from BeautifulSoup import Comment
@@ -149,48 +150,9 @@ def get_page_base(page, html):
     return page.geturl()
 
 
-def get_page_html_only(url, **kwargs):
-    """ Retrieves and parses html page.
-
-    kwargs are passed to Request object constructor.
-    """
-    page = urlopener.open(urllib2.Request(url=quote_url_path(url), **kwargs))
-    html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
-    base = get_page_base(page, html)
-    return (page, html, base)
-
-
 refresh_reqex = re.compile("refresh", re.IGNORECASE)
 
 
-def try_skip_banner(page, html, base, **kwargs):
-    """ This may help avoid ads on some resources.
-
-    When web page contains Refresh in its meta it is likely that 
-    it is banner so we try to reload it immediately.
-    """
-    refresh = None
-    if page.info().has_key("Refresh"):
-        refresh = page.info()["Refresh"]
-    else:
-        tag = html.find(name = "meta", attrs = {"http-equiv": refresh_reqex})
-        if tag:
-            refresh = tag["content"]
-    if refresh:
-        new_url = page.geturl()
-        ind = refresh.lower().find("url=")
-        if ind >= 0:
-            new_url = urljoin(base, refresh[ind + 4:])
-        return get_page_html_only(new_url, **kwargs)
-    return (page, html, base)
-
-
-def get_page_html(url, **kwargs):
-    (page, html, base) = get_page_html_only(url, **kwargs)
-    #(page, html, base) = try_skip_banner(page, html, base, **kwargs)
-    return (html, base, page.geturl())
-
-
 def get_feed_title(xml):
     chan = xml.find(["channel", "feed"])
     if chan:
@@ -252,6 +214,40 @@ def discover_feed(url):
     return (url, title, icon_url)
 
 
+class Downloader:
+    """ This class helps managing multithreading downloads. """
+
+    def __init__(self, callback):
+        self.__threads = []
+        self.__callback = callback
+        self.__cached = {}
+
+
+    def queue(self, url, abs_path, rel_path):
+        if self.__cached.has_key(url):
+            return self.__cached[url]
+        filename = uuid.uuid1().hex + urllib.unquote(urlparse(url)[2].split("/")[-1])
+        path_for_url = os.path.join(rel_path, filename)
+        self.__cached[url] = path_for_url
+        thread = Thread(target = self.__retrieve, args = (quote_url_path(url), os.path.join(abs_path, filename)))
+        self.__threads.append(thread)
+        thread.start()
+        return path_for_url
+
+
+    def __retrieve(self, url, filename):
+        try:
+            urllib.urlretrieve(url, filename)
+        except Exception, ex:
+            if self.__callback:
+                self.__callback(("error", ex))
+
+
+    def wait_completion(self):
+        for thread in self.__threads:
+            thread.join()
+
+
 class Item:
     """ This class groups feed element related operations together.
 
@@ -266,7 +262,8 @@ class Item:
 
     def __init__(self, url = None, tag = None, base = None, path = "", cache_path = "", callback = None, \
         page_template = None, print_version_keywords = None, try_use_print_version = True, \
-        allow_scripts = False, as_is = False, content_kwargs = None, reformatter_url = None, next_page_keyword = None):
+        allow_scripts = False, as_is = False, content_kwargs = None, reformatter_url = None, next_page_keyword = None, \
+        downloader = None):
         """ It is clear that this is constructor :)
 
         Item is represented either by its content, when tag is specified, or by reference
@@ -292,7 +289,6 @@ class Item:
         self.__base = base
         self.__path = path
         self.__cache_path = cache_path
-        self.__cached = {}
         self.__try_use_print_version = try_use_print_version
         self.__allow_scripts = allow_scripts
         self.__page_template = self.page_template_default
@@ -311,6 +307,9 @@ class Item:
         self.__reformatter_url = reformatter_url
         self.__next_page_keyword = next_page_keyword
         self.__next_page_a = None
+        self.__downloader = downloader
+        if not self.__downloader:
+            self.__downloader = Downloader(callback)
 
 
     def __report(self, action, item):
@@ -348,6 +347,19 @@ class Item:
         return cache_path
 
 
+    def __get_page_html(self, url, **kwargs):
+        """ Retrieves and parses html page.
+
+        kwargs are passed to Request object constructor.
+        """
+        self.__report("downloading", url)
+        page = urlopener.open(urllib2.Request(url=quote_url_path(url), **kwargs))
+        self.__report("parsing", url)
+        html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
+        base = get_page_base(page, html)
+        return (html, base, page.geturl())
+
+
     def __cache_next_page(self, html, base):
         try:
             text = html.find(text=lambda t: t.strip().lower() == self.__next_page_keyword.strip().lower())
@@ -373,8 +385,7 @@ class Item:
                 a = text.findParent(lambda t: t.name == "a" and t.has_key("href"))
                 if a:
                     url = urljoin(base, a["href"])
-                    self.__report("downloading", url)
-                    (new_html, new_base, real_url) = get_page_html(url, headers={"Referer": referer})
+                    (new_html, new_base, real_url) = self.__get_page_html(url, headers={"Referer": referer})
                     return (new_html, new_base, new_html.body)
         except Exception, ex:
             self.__report("error", ex)
@@ -392,8 +403,7 @@ class Item:
 
         By extracting html document and reformatting it.
         """
-        self.__report("downloading", self.__url)
-        (html, self.__base, real_url) = get_page_html(self.get_url_for_reformatter(self.__url))
+        (html, self.__base, real_url) = self.__get_page_html(self.get_url_for_reformatter(self.__url))
         if not self.__as_is:
             if self.__try_use_print_version:
                 (html, self.__base, self.__tag) = self.__find_print_version(html, self.__base, real_url)
@@ -428,16 +438,9 @@ class Item:
         if tag.has_key(attr):
             try:
                 url = urljoin(base, tag[attr])
-                path_for_url = None
-                if self.__cached.has_key(url):
-                    path_for_url = self.__cached[url]
-                else:
-                    filename = uuid.uuid1().hex + urllib.unquote(urlparse(url)[2].split("/")[-1])
-                    path_for_url = os.path.join(self.__cache_path, filename)
-                    tag.parent.append(Comment(NavigableString(self.cached_comment_prefix + path_for_url)))
-                    self.__report("downloading", url)
-                    urllib.urlretrieve(quote_url_path(url), os.path.join(self.__get_cache_directory(), filename))
-                    self.__cached[url] = path_for_url
+                path_for_url = self.__downloader.queue(url, 
+                    self.__get_cache_directory(), self.__cache_path)
+                tag.parent.append(Comment(NavigableString(self.cached_comment_prefix + path_for_url)))
                 tag[attr] = path_for_url
             except Exception, ex:
                 self.__report("error", ex)
@@ -496,9 +499,14 @@ class Item:
             if self.__url:
                 self.__extract_tag()
             self.__contents = self.__fixup_tag()
-        except Exception, ex:
+        except BaseException, ex:
             self.__report("error", ex)
+            self.__downloader.wait_completion()
             self.delete()
+            if isinstance(ex, Exception):
+                self.__contents.append(NavigableString(str(ex)))
+            else:
+                raise
         return self.__contents
 
 
@@ -593,6 +601,7 @@ class Feed:
         self.__content_kwargs = content_kwargs
         self.__reformatter_url = reformatter_url
         self.__next_page_keyword = next_page_keyword
+        self.__downloader = Downloader(callback)
 
 
     def __report(self, action, item):
@@ -669,7 +678,8 @@ class Feed:
             callback = self.__callback, try_use_print_version = self.__try_use_print_version, \
             print_version_keywords = self.__print_version_keywords, allow_scripts = self.__allow_scripts, \
             as_is = self.__as_is, content_kwargs = self.__content_kwargs, \
-            reformatter_url = self.__reformatter_url, next_page_keyword = self.__next_page_keyword)
+            reformatter_url = self.__reformatter_url, next_page_keyword = self.__next_page_keyword, \
+            downloader = self.__downloader)
         
 
     def __add_item(self, div, item):
@@ -730,6 +740,7 @@ class Feed:
             self.__report("downloading", self.__url)
             page = urlopener.open(quote_url_path(self.__url))
             self._real_url = page.geturl()
+            self.__report("parsing", self.__url)
             xml = BeautifulStoneSoup(page, markupMassage = massage)
             if not self.__title:
                 self.__title = get_feed_title(xml)
@@ -756,11 +767,14 @@ class Feed:
             result = self.__get_new_item_count(div)
             if result:
                 self.__delete_before(html)
+                self.__report("downloading images", "")
+                self.__downloader.wait_completion()
                 f = open(path, "w")
                 f.write(html.prettify())
                 f.close()
-        except Exception, ex:
+        except BaseException, ex:
             self.__report("error", ex)
+            self.__downloader.wait_completion()
             if div:
                 self.__delete_div(div)
         return result
