@@ -18,13 +18,14 @@ import urllib
 import webbrowser
 import glob
 import shutil
+import sys
 
 gtk.gdk.threads_init()
 
 #This is to be able to start Feed Circuit on PC
 try:
     import osso
-    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.7", False)
+    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.8", False)
 except ImportError:
     pass
 
@@ -159,6 +160,7 @@ class FeedConfig:
         self.icon = None
         self.iter = None
         self.ord_num = 0
+        self.checked = False
 
 
     def get_icon(self):
@@ -215,7 +217,7 @@ class FeedListModel(gtk.GenericTreeModel):
 
     def __init__(self, config_path):
         gtk.GenericTreeModel.__init__(self)
-        self.column_types = (gtk.gdk.Pixbuf, str, str)
+        self.column_types = (gtk.gdk.Pixbuf, str, str, gobject.TYPE_BOOLEAN)
         self.list = []
         self.map = {}
         self.config_path = config_path
@@ -252,6 +254,8 @@ class FeedListModel(gtk.GenericTreeModel):
                 return self.list[self.map[rowref]].get_markup()
             elif column is 2:
                 return self.list[self.map[rowref]].get_time()
+            elif column is 3:
+                return self.list[self.map[rowref]].checked
 
 
     def on_iter_next(self, rowref):
@@ -291,6 +295,7 @@ class FeedListModel(gtk.GenericTreeModel):
         parser = ConfigParserEx()
         parser.read([os.path.join(self.config_path, "config")])
         self.cache_path = parser.get_default("global", "cache path", self.config_path)
+        self.use_checkboxes = parser.getboolean_default("global", "use checkboxes", False)
         for section in parser.sections():
             if section != "global":
                 feed_config = FeedConfig(
@@ -322,6 +327,7 @@ class FeedListModel(gtk.GenericTreeModel):
             parser = ConfigParserEx()
             parser.add_section("global")
             parser.set("global", "cache path", self.cache_path)
+            parser.set("global", "use checkboxes", str(self.use_checkboxes))
             i = 0
             for feed_config in self.list:
                 parser.add_section(feed_config.id)
@@ -400,6 +406,19 @@ class FeedListModel(gtk.GenericTreeModel):
         self.save_config()
 
 
+    def toggle(self, path):
+        feed_config = self.list[path[0]]
+        feed_config.checked = not feed_config.checked
+        self.row_changed(path, self.get_iter(path))
+
+
+    def set_checkboxes(self, value):
+        self.use_checkboxes = value
+        if not self.use_checkboxes:
+            for feed_config in self.list:
+                feed_config.checked = False
+
+
 class UrlsWindow(gtk.Dialog):
 
 
@@ -437,12 +456,14 @@ class PreferencesWindow(gtk.Dialog):
         self.choose_button = gtk.ToolButton(stock_id = gtk.STOCK_DIRECTORY)
         self.choose_button.connect("clicked", self.choose_button_clicked)
         hbox.pack_start(self.choose_button, expand = False)
-        table = gtk.Table(rows = 1, columns = 2)
-        texts = [_("Cache path")]
+        self.checkboxes_check = gtk.CheckButton()
+        table = gtk.Table(rows = 2, columns = 2)
+        texts = [_("Cache path"), _("Show checkboxes")]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
             table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
         table.attach(hbox, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
+        table.attach(self.checkboxes_check, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
         self.vbox.add(table)
         self.show_all()
 
@@ -725,6 +746,15 @@ class Program(hildon.Program):
         self.__view.set_rubber_banding(True)
         self.__view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
 
+        self.check_col = gtk.TreeViewColumn()
+        render_toggle = gtk.CellRendererToggle()
+        render_toggle.set_property("activatable", True)
+        render_toggle.connect("toggled", self.feed_toggled)
+        self.check_col.pack_start(render_toggle, expand = False)
+        self.check_col.add_attribute(render_toggle, "active", 3)
+        self.__view.append_column(self.check_col)
+        self.check_col.set_visible(self.__model.use_checkboxes)
+
         col = gtk.TreeViewColumn()
         render_pixbuf = gtk.CellRendererPixbuf()
         col.pack_start(render_pixbuf, expand=False)
@@ -770,6 +800,7 @@ class Program(hildon.Program):
             button.connect("clicked", callback)
             toolbar.insert(button, -1)
 
+
     def on_key_press(self, widget, event, *args):
         if event.keyval == gtk.keysyms.F6:
             # The "Full screen" hardware key has been pressed
@@ -793,10 +824,9 @@ class Program(hildon.Program):
 
 
     def drag_data_get(self, widget, drag_context, selection_data, info, timestamp):
-        feed_configs = self.get_selected_feeds()
-        if len(feed_configs):
-            id = feed_configs[0].id
-            selection_data.set(selection_data.target, len(id), id)
+        feed_config = self.get_current_feed()
+        if feed_config:
+            selection_data.set(selection_data.target, len(feed_config.id), feed_config.id)
 
 
     def drag_data_received(self, widget, drag_context, x, y, selection_data, info, timestamp):
@@ -810,6 +840,10 @@ class Program(hildon.Program):
         self.__model.move(selection_data.data, pos)
 
 
+    def feed_toggled(self, cell, path):
+        self.__model.toggle((int(path),))
+
+
     def run(self):
         self.__window.show_all()
         gtk.main()
@@ -834,12 +868,22 @@ class Program(hildon.Program):
 
 
     def get_selected_feeds(self):
+        if self.__model.use_checkboxes:
+            return filter(lambda fc: fc.checked, self.__model.list)
+        else:
+            pathes = self.__view.get_selection().get_selected_rows()[1]
+            return map(lambda path: self.__model.list[path[0]], pathes)
+
+
+    def get_current_feed(self):
         pathes = self.__view.get_selection().get_selected_rows()[1]
-        return map(lambda path: self.__model.list[path[0]], pathes)
+        if len(pathes):
+            return self.__model.list[pathes[0][0]]
 
 
     def open(self, source):
-        for feed_config in self.get_selected_feeds():
+        feed_config = self.get_current_feed()
+        if feed_config:
             self.open_feed(feed_config)
 
 
@@ -876,9 +920,10 @@ class Program(hildon.Program):
     def callback(self, item):
         self.__abort_lock.acquire()
         abort = self.__aborted
+        self.__aborted = False
         self.__abort_lock.release()
         if abort:
-            exit()
+            sys.exit()
         text = "(" + str(self.__progress_info[0]) + "/" + str(self.__progress_info[1]) + ") " + \
             _(str(item[0])) + " " + str(item[1])
         gobject.idle_add(self.update_status, text)
@@ -896,6 +941,7 @@ class Program(hildon.Program):
                 i += 1
                 self.__progress_info = (i, len(feed_configs))
                 urls = feed_config.get_urls()
+                new_item_count = 0
                 title = None
                 if len(urls) == 1:
                     title = feed_config.name
@@ -910,8 +956,8 @@ class Program(hildon.Program):
                         as_is = feed_config.as_is, content_kwargs = feed_config.get_content_kwargs(),
                         print_version_keywords = feed_config.get_print_version_keywords(),
                         reformatter_url = feed_config.reformatter_url, next_page_keyword = feed_config.next_page_link)
-                    new_item_count = feed.update()
-                    gobject.idle_add(self.feed_updated, feed_config, new_item_count)
+                    new_item_count += feed.update()
+                gobject.idle_add(self.feed_updated, feed_config, new_item_count)
         finally:
             gobject.idle_add(self.update_complete)
             self.__update_lock.release()
@@ -931,9 +977,8 @@ class Program(hildon.Program):
 
 
     def edit(self, source):
-        feed_configs = self.get_selected_feeds()
-        if len(feed_configs):
-            feed_config = feed_configs[0]
+        feed_config = self.get_current_feed()
+        if feed_config:
             edit_window = EditWindow(self.__window, feed_config)
             if edit_window.run() == gtk.RESPONSE_ACCEPT:
                 edit_window.update_feed()
@@ -1023,11 +1068,14 @@ class Program(hildon.Program):
         pref_window = PreferencesWindow(self.__window)
         try:
             pref_window.path_entry.set_text(self.__model.cache_path)
+            pref_window.checkboxes_check.set_active(self.__model.use_checkboxes)
             if pref_window.run() == gtk.RESPONSE_ACCEPT:
                 new_path = os.path.abspath(pref_window.path_entry.get_text())
                 if new_path != self.__model.cache_path:
                     self.move_cache(self.__model.cache_path, new_path)
                     self.__model.update_cache_path(new_path)
+                self.__model.set_checkboxes(pref_window.checkboxes_check.get_active())
+                self.check_col.set_visible(self.__model.use_checkboxes)
                 self.__model.save_config()
         except Exception, ex:
             msg_dlg = gtk.MessageDialog(parent = self.__window, flags = gtk.DIALOG_MODAL,
@@ -1042,7 +1090,7 @@ class Program(hildon.Program):
     def about(self, source):
         dialog = gtk.AboutDialog()
         dialog.set_name("FeedCircuit")
-        dialog.set_version("0.6.7")
+        dialog.set_version("0.6.8")
         dialog.set_website("http://feedcircuit.garage.maemo.org")
         dialog.set_authors([_("George Kibardin")])
         dialog.run()
